package com.androidphonesignal;



import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpConnectionParams;

import com.androidphonesignal.R;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Environment;
import android.telephony.PhoneStateListener;
import android.telephony.SignalStrength;
import android.telephony.TelephonyManager;
import android.telephony.gsm.GsmCellLocation;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.Toast;



public class AndroidSignalActivity extends MapActivity implements SensorEventListener{
    /** Called when the activity is first created. */
	@SuppressWarnings("unused")
	private static final String TAG = "AndroidSignalActivity";
	
	// attributes used by location service
    private LocationListener myLocLis;
    private LocationManager lm;
    private String bestLocationProvider;
    private Criteria criteria;
    private MapView map;
    private MapController mapController;
    private List<Overlay> mapOverlays;
    private Drawable drawable;
    private MyItemizedOverlay itemizedOverlay;
    private ImageButton imgBtnLoc;
    private boolean imgBtnLocEnabled;
    private Location myLocation;
    
    // attributes for light acquisition
    private SensorManager mySensorManager;
    private Sensor myLightSensor;
    private boolean imgBtnLightEnabled;
    private float myLightValue;
    private long timestamp;
    
    // attributes for phone signal acquisition
    private TelephonyManager myTelephonyManager;
    private String[] networkType = {"UNKNOWN", "GPRS", "EDGE", "UMTS", "CDMA", "EVDO_0", "EVDO_A", "RTT", "HSDPA", "HSUPA", "HSPA", "IDEN", "EVDO_B"};
    private SignalStrength mySignalStrength;
    private PhoneStateListener myPhoneStateListener;
    private boolean imgBtnSignal = true;
    private Sensor myOrientationSensor;
    private int countOrientation;
    private int signalSample;
    private int signalValue;
    private ProgressDialog myProgressSignal;
    private int sampleNetworkType;
    private float prevValue;
    
    public AndroidSignalActivity() {
    	prevValue = 0;
    }
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.main);
        
        // create folder for reports in the external storage
        if (!new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal").exists())
        	new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal").mkdir();
        if (!new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport").exists())
        	new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport").mkdir();
        
        // location informations
        imgBtnLocEnabled = true;
        imgBtnLoc = ((ImageButton) findViewById(R.id.btnGetPos));
        lm = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
        map = (MapView) findViewById(R.id.mapview);
        mapController = map.getController();
        mapOverlays = map.getOverlays();
        drawable = this.getResources().getDrawable(R.drawable.androidmarker);
        itemizedOverlay = new MyItemizedOverlay(drawable);
        
        // behavior for the location button
        imgBtnLoc.setOnClickListener(new OnClickListener() {
        	public void onClick(View arg0) {
        		if (!imgBtnLocEnabled) {
        			lm.removeUpdates(myLocLis);
        			myLocLis = null;
        			imgBtnLocEnabled = true;
        			Toast.makeText(getApplicationContext(), "Position retrieval disabled!", Toast.LENGTH_SHORT).show();
        		}
        		else {
	                myLocLis = new MyLocationListener();
	                criteria = new Criteria();
	                criteria.setAccuracy(Criteria.ACCURACY_FINE); // ACCURACY_LOW for Android 2.3.3 API
	                criteria.setCostAllowed(true);
	                criteria.setSpeedRequired(false);
	                criteria.setPowerRequirement(Criteria.POWER_MEDIUM);
	        		bestLocationProvider = lm.getBestProvider(criteria, true);
	                lm.requestLocationUpdates(bestLocationProvider, 5000, 5, myLocLis);
	                Toast.makeText(getApplicationContext(),"Position provided by: " + bestLocationProvider, Toast.LENGTH_SHORT).show();
	                imgBtnLocEnabled = false;
        		}
        	}
        });
        
        // light informations
        myLightValue = -1;
        mySensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        myLightSensor = mySensorManager.getDefaultSensor(Sensor.TYPE_LIGHT);
        mySensorManager.unregisterListener(this);
        imgBtnLightEnabled = true;
        
        // behavior for the light button
        ((ImageButton) findViewById(R.id.imgBtnLight)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (imgBtnLightEnabled) {
					mySensorManager.registerListener(AndroidSignalActivity.this, myLightSensor, SensorManager.SENSOR_DELAY_NORMAL);
        			Toast.makeText(getApplicationContext(), "Light informations enabled!", Toast.LENGTH_SHORT).show();
					imgBtnLightEnabled = false;
				}
				else {
			         mySensorManager.unregisterListener(AndroidSignalActivity.this, myLightSensor);
			         Toast.makeText(getApplicationContext(), "Light informations disabled!", Toast.LENGTH_SHORT).show();
			         imgBtnLightEnabled = true;
				}
			}
		});
        
        // phone signal informations
        signalValue = -1;
        mySensorManager.unregisterListener(this);
        myOrientationSensor = mySensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION);
		myTelephonyManager = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
		myProgressSignal = new ProgressDialog(AndroidSignalActivity.this);
		myProgressSignal.setMax(360);
		myProgressSignal.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		myProgressSignal.setTitle("Acquiring signal...");
		
        // behavior for the phone signal button
        ((ImageButton) findViewById(R.id.imgBtnSignal)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (myTelephonyManager.getSimState() == TelephonyManager.SIM_STATE_READY) {
					if (imgBtnSignal) {
						sampleNetworkType = myTelephonyManager.getNetworkType();
						mySensorManager.registerListener(AndroidSignalActivity.this, myOrientationSensor, SensorManager.SENSOR_ORIENTATION);
				        myPhoneStateListener = new MyPhoneStateListener();
				        countOrientation = 0;
						signalSample = 0;
						myProgressSignal.setProgress(0);
						Toast.makeText(getApplicationContext(), "Orientation informations enabled!", Toast.LENGTH_SHORT).show();
						myTelephonyManager.listen(myPhoneStateListener, PhoneStateListener.LISTEN_SIGNAL_STRENGTHS);
						myProgressSignal.show();
						Toast.makeText(getApplicationContext(), "Signal strength informations enabled!", Toast.LENGTH_SHORT).show();
						imgBtnSignal = false;
					}
					else {
						timestamp = System.currentTimeMillis();
				        mySensorManager.unregisterListener(AndroidSignalActivity.this, myOrientationSensor);
						myTelephonyManager.listen(myPhoneStateListener, PhoneStateListener.LISTEN_NONE);
						myPhoneStateListener = null;
						Toast.makeText(getApplicationContext(), "Signal strength informations disabled!", Toast.LENGTH_SHORT).show();
						imgBtnSignal = true;
						Toast.makeText(getApplicationContext(), "Orientation informations disabled!", Toast.LENGTH_SHORT).show();
					}
				} 
				else {
					Toast.makeText(getApplicationContext(), "No network available!", Toast.LENGTH_LONG).show();
				}
			} 
		});
        
        // behavior for the report button
        ((ImageButton) findViewById(R.id.imgBtnTransfer)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent i = new Intent(AndroidSignalActivity.this, ReportTransferActivity.class);
				if (myTelephonyManager == null || signalValue == -1 || myLocation == null)
					Toast.makeText(getApplicationContext(), "No sufficient informations to report!", Toast.LENGTH_SHORT).show();
				else {
					i.putExtra("OperatorName", myTelephonyManager.getNetworkOperatorName());
					i.putExtra("SignalStrength", signalValue);
					i.putExtra("NetworkType", networkType[sampleNetworkType]);
					i.putExtra("Roaming", myTelephonyManager.isNetworkRoaming());
					i.putExtra("CellID", String.valueOf(((GsmCellLocation)myTelephonyManager.getCellLocation()).getCid() & 0xffff));
					i.putExtra("LightInfo", myLightValue);
					i.putExtra("Latitude", myLocation.getLatitude());
					i.putExtra("Longitude", myLocation.getLongitude());
					i.putExtra("LocationProvider", bestLocationProvider);
					i.putExtra("Timestamp", timestamp);
					startActivity(i);
				}
			}
		});
        
        // behavior for the upload button
        ((ImageButton) findViewById(R.id.imgBtnUpload)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v){
				// archive all files and send it to the server
				HttpResponse resp;
				// zip all data to transfer: it creates only one file 
				File androidPhoneSignalDir = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal");
				File myReportDir = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport");
				String zipFileName = Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/archive_" + System.currentTimeMillis() + ".zip";
				ZipOutputStream zos = null;
				try {
					zos = new ZipOutputStream(new FileOutputStream(zipFileName));
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				}
				 try {
					 // lists all files in the AndroidPhoneSignal directory
				     for (int i = 0; i < androidPhoneSignalDir.list().length; i++) {
				    	 // if it is a file, add it to the archive
				    	 if (androidPhoneSignalDir.listFiles()[i].isFile() && !androidPhoneSignalDir.listFiles()[i].getName().endsWith(".zip"))
				    	 {
					         String filename = androidPhoneSignalDir.list()[i];
					         byte[] bytes = new byte[(int)androidPhoneSignalDir.listFiles()[i].length()];
					         new FileInputStream(androidPhoneSignalDir.listFiles()[i]).read(bytes);
					         ZipEntry entry = new ZipEntry(filename);
					         zos.putNextEntry(entry);
					         zos.write(bytes);
					         zos.closeEntry();
				    	 }
				    	 // if it is a directory, grab the files from that and put them in the archive
				    	 if (androidPhoneSignalDir.listFiles()[i].isDirectory())
				    	 {
				    		 File dir = androidPhoneSignalDir.listFiles()[i];
				    		 byte[] bytes;
				    		 ZipEntry entry;
					         for (int k = 0; k < dir.list().length; k++)
					         {
					        	 if (dir.listFiles()[k].isFile())
					        	 {
						        	 String filename = dir.list()[k];
							         bytes = new byte[(int)dir.listFiles()[k].length()];
							         new FileInputStream(dir.listFiles()[k]).read(bytes);
							         entry = new ZipEntry(dir.getName() + "/" + filename);
							         zos.putNextEntry(entry);
							         zos.write(bytes);
							         zos.closeEntry();
					        	 }
					         }
				    	 }
				     }
				 } catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
				     try {
						zos.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				 }
				 
				File zipFile = new File(zipFileName);
				if (zipFile.length() != 0) {
					HttpClient httpClient = new DefaultHttpClient();
					httpClient.getParams().setParameter(HttpConnectionParams.CONNECTION_TIMEOUT, 10000);
					httpClient.getParams().setParameter(HttpConnectionParams.SO_TIMEOUT, 15000);
					httpClient.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);
					HttpPost httpPost = new HttpPost("http://192.168.0.3:8080/ProgettoJDBC/SrvHello");
					MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
					entity.addPart("reportFile", new FileBody(zipFile));
					httpPost.setEntity(entity);
					Toast.makeText(getApplicationContext(), "Sending all reports...", Toast.LENGTH_SHORT).show();
					try {
						resp = httpClient.execute(httpPost);
						if (resp.getStatusLine().getStatusCode() == 200) {
							zipFile.delete();
							// delete all files transferred
							File[] app1 = androidPhoneSignalDir.listFiles();
							File[] app2 = myReportDir.listFiles();
							int l = app1.length;
							for (int i = 0; i < l; i++) {
								if (app1[i].isFile())
									app1[i].delete();
							}
							l = app2.length;
							for (int i = 0; i < l; i++) {
								app2[i].delete();
							}
						}
						else {
							zipFile.delete();
						}
						Toast.makeText(getApplicationContext(), resp.getStatusLine().getStatusCode()==200?"File sent successfully!":"Transfer error. Retry!", Toast.LENGTH_LONG).show();
					} catch (ClientProtocolException e) {
						if (zipFile.exists()) {
							zipFile.delete();
						}
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					} catch (ParseException e) {
						if (zipFile.exists()) {
							zipFile.delete();
						}
						e.printStackTrace();
					} catch (IOException e) {
						if (zipFile.exists()) {
							zipFile.delete();
						}
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					} catch (Exception e) {
						if (zipFile.exists()) {
							zipFile.delete();
						}
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					}
				}
				else {
					zipFile.delete();
					Toast.makeText(getApplicationContext(), "No reports to send!", Toast.LENGTH_LONG).show();
				}
			}
		});
    }
    @Override
    protected void onPause() {
    	super.onPause();
        mySensorManager.unregisterListener(this);
        if (myTelephonyManager != null && myPhoneStateListener != null)
        	myTelephonyManager.listen(myPhoneStateListener, PhoneStateListener.LISTEN_NONE);
    	if (myLocLis != null)
    		lm.removeUpdates(myLocLis);
    }
    @Override
    protected void onStop() {
    	super.onStop();
        mySensorManager.unregisterListener(this);
        if (myTelephonyManager != null && myPhoneStateListener != null)
        	myTelephonyManager.listen(myPhoneStateListener, PhoneStateListener.LISTEN_NONE);
    	if (myLocLis != null)
        	lm.removeUpdates(myLocLis);
    }
    
    @Override
    public void onResume() {
    	super.onResume();
        if (lm != null && myLocLis != null){
			bestLocationProvider = lm.getBestProvider(criteria, true);
	        lm.requestLocationUpdates(bestLocationProvider, 5000, 5, myLocLis);
        }
        if (myTelephonyManager != null && myPhoneStateListener != null)
        	myTelephonyManager.listen(myPhoneStateListener, PhoneStateListener.LISTEN_SIGNAL_STRENGTHS);
        if (!imgBtnSignal)
        	mySensorManager.registerListener(AndroidSignalActivity.this, myOrientationSensor, Sensor.TYPE_ORIENTATION);
        if (!imgBtnLightEnabled)
        	mySensorManager.registerListener(AndroidSignalActivity.this, myLightSensor, Sensor.TYPE_LIGHT);
    }

// override method used by MapActivity
@Override
protected boolean isRouteDisplayed() {
	return false;
}

// override method used by SensorEventListener
@Override
public void onAccuracyChanged(Sensor sensor, int accuracy) {
}

// override method used by SensorEventListener
@Override
public void onSensorChanged(SensorEvent event) {
	// action for light sensor value changed
	if (event.sensor.getType() == Sensor.TYPE_LIGHT){
		Toast.makeText(getApplicationContext(),"Light value: " + String.valueOf(event.values[0]), Toast.LENGTH_SHORT).show();
		myLightValue = event.values[0];
	}
	// action for orientation value changed
	if (event.sensor.getType() == Sensor.TYPE_ORIENTATION) {
		// if the type of the network is changed meanwhile, stop the acquirement and do the average 
		if (sampleNetworkType == myTelephonyManager.getNetworkType()) {
			if (prevValue == 0) {
				countOrientation++;
				prevValue = event.values[0];
			}
			else
				if (countOrientation <= 360 && event.values[0]!=prevValue) {
					countOrientation++;
					signalSample += mySignalStrength.getGsmSignalStrength();
					prevValue = event.values[0];
					myProgressSignal.setProgress(countOrientation);
				}
			if (countOrientation == 360) {
				signalValue = signalSample/countOrientation;
				Toast.makeText(getApplicationContext(),String.valueOf(signalValue), Toast.LENGTH_SHORT).show();
				prevValue = 0;
				myProgressSignal.dismiss();
				mySensorManager.unregisterListener(AndroidSignalActivity.this, myOrientationSensor);
			}
		}
		else {
			if (countOrientation <= 360 && countOrientation>0) {
				signalValue=signalSample/countOrientation;
				Toast.makeText(getApplicationContext(),String.valueOf(signalValue), Toast.LENGTH_SHORT).show();
				prevValue = 0;
				myProgressSignal.dismiss();
				mySensorManager.unregisterListener(AndroidSignalActivity.this, myOrientationSensor);
			}
		}
	}
}

class MyLocationListener implements LocationListener {    
    @Override
	public void onLocationChanged(Location location) {
		String Text = "My current NEW location is: " + "\nLatitude = " + location.getLatitude() + "\nLongitude = " + location.getLongitude();
		Toast.makeText(getApplicationContext(), Text, Toast.LENGTH_LONG).show();
		mapController.setZoom(17);
		GeoPoint point = new GeoPoint((int) (location.getLatitude() * 1E6), (int) (location.getLongitude() * 1E6));
		mapController.animateTo(point);
		OverlayItem overlayitem = new OverlayItem(point, "", "");
		itemizedOverlay.clear();
		itemizedOverlay.addOverlay(overlayitem);
		mapOverlays.add(itemizedOverlay);
		myLocation = location;
	}

	@Override
	public void onProviderDisabled(String provider) {
		Toast.makeText(getApplicationContext(), provider + " Disabled", Toast.LENGTH_SHORT).show();
	}

	@Override
	public void onProviderEnabled(String provider) {
		Toast.makeText(getApplicationContext(), provider + " Enabled", Toast.LENGTH_SHORT).show();
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
	}
}

private class MyPhoneStateListener extends PhoneStateListener
{
  // get the signal strength from the provider, each time there is an update
  private int signal;
  @Override
  public void onSignalStrengthsChanged(SignalStrength signalStrength)
  {
     super.onSignalStrengthsChanged(signalStrength);
     if (myTelephonyManager.getSimState() == TelephonyManager.SIM_STATE_READY) {
	     mySignalStrength = signalStrength;
	     signal = signalStrength.getGsmSignalStrength();
	     
	     Toast.makeText(getApplicationContext(), "Network info:\n" + "Operator name: " 
					+ myTelephonyManager.getNetworkOperatorName() 
					+ "\nNetwork type: " + networkType[myTelephonyManager.getNetworkType()]
					+ "\nGSM Signal strength: " + String.valueOf(signal + " asu" + " (" + String.valueOf(-113+2*signal) + " dBm)")
					+ "\nRoaming: " + String.valueOf(myTelephonyManager.isNetworkRoaming()) 
					+ "\nCell ID: " + String.valueOf(((GsmCellLocation)myTelephonyManager.getCellLocation()).getCid() & 0xffff), Toast.LENGTH_SHORT).show(); // 0xffff mask used when connected to 3G cells
     }
  }
}
}