package com.androidphonesignal;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Set;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.google.gson.Gson;


import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

public class BluetoothActivity extends Activity{
	
    private static final int REQUEST_ENABLE_BT = 0;
    private static final String TAG = "BluetoothActivity";
    
	// attributes for ad-hoc connection
    private ArrayAdapter<String> myArrayAdapter;
    private BroadcastReceiver mReceiver;
    private BluetoothAdapter myBluetoothAdapter;
    private BluetoothSocket mySocket;
    private IntentFilter filter;
    private UUID uuid;
    private ListView l;
    private Bundle extras;
    private Gson signalReportGson;
    private SignalReportTO mySignalReport;
    private String jsonSignalReport;
    
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setTitle("Trasferimento Bluetooth");
        setContentView(R.layout.bttransfer);
        extras = getIntent().getExtras();
        signalReportGson = new Gson();
        mySignalReport = new SignalReportTO();
        // UUID for the application
        uuid = UUID.fromString("d2c148c5-6f4f-46c6-8209-2eb2c7e023ef");

        // assign values to SignalReportTO: it contains the data to be transferred
        mySignalReport.setAvgSignalStrength(extras.getInt("SignalStrength"));
        mySignalReport.setCellID(extras.getString("CellID"));
        mySignalReport.setLatitude(extras.getDouble("Latitude"));
        mySignalReport.setLightInfo(extras.getFloat("LightInfo"));
        mySignalReport.setLongitude(extras.getDouble("Longitude"));
        mySignalReport.setNetworkType(extras.getString("NetworkType"));
        mySignalReport.setOperatorName(extras.getString("OperatorName"));
        mySignalReport.setRoaming(extras.getBoolean("Roaming"));
        mySignalReport.setLocationProvider(extras.getString("LocationProvider"));
        mySignalReport.setTimestamp(extras.getLong("Timestamp"));
        
        jsonSignalReport = signalReportGson.toJson(mySignalReport);
        Toast.makeText(getApplicationContext(), jsonSignalReport, Toast.LENGTH_LONG).show();
        
        l = ((ListView)findViewById(R.id.textPairedBTDevices));
        
        // ad-hoc connection provided by Bluetooth
        myBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        
        if (!myBluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }
   	  
   	// Create a BroadcastReceiver for ACTION_FOUND: a new Bluetooth device has been discovered
 	mReceiver = new BroadcastReceiver() {
 	    public void onReceive(Context context, Intent intent) {
 	        String action = intent.getAction();
 	        // When discovery finds a device
 	        if (BluetoothDevice.ACTION_FOUND.equals(action)) {
 	            // Get the BluetoothDevice object from the Intent
 	            BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
 	            // Add the name and address to an array adapter to show in a ListView and check if already present
 	            Toast.makeText(getApplicationContext(), String.valueOf(myArrayAdapter.getPosition(device.getName() + "\n" + device.getAddress())), Toast.LENGTH_LONG).show();
 	            if (myArrayAdapter.getPosition(device.getName() + "\n" + device.getAddress())<0 && device.getName()!=null)
 	            	myArrayAdapter.add(device.getName() + "\n" + device.getAddress());
 	        }
 	    }
 	};
 		// Register the BroadcastReceiver inside the system
 		filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
 		registerReceiver(mReceiver, filter); // Don't forget to unregister during onDestroy
 		
 		Set<BluetoothDevice> pairedDevices = myBluetoothAdapter.getBondedDevices();
        myArrayAdapter = new ArrayAdapter<String>(this,R.layout.list_item);
         
        // Loop through paired devices
         for (BluetoothDevice device : pairedDevices) {
             // Add the name and address to an array adapter to show in a ListView
             myArrayAdapter.add(device.getName() + "\n" + device.getAddress());
         }
         l.setAdapter(myArrayAdapter);
 		
         // behavior for the "run bluetooth server" button
 		((Button)findViewById(R.id.btnRunServer)).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				AcceptThread at = new AcceptThread();
				at.start();
			}
		});
 		
        // behavior for the "discovery devices" button
 		((Button)findViewById(R.id.btnDiscovery)).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				myBluetoothAdapter.startDiscovery();
			}
		});
 		
        // behavior for the "set discoverable" button
		((Button)findViewById(R.id.btnSetDiscoverable)).setOnClickListener(new View.OnClickListener() {	
			@Override
			public void onClick(View v) {
				Intent discoverableIntent = new
		        Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
				// set the device discoverable for 300 seconds
		        discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
		        startActivity(discoverableIntent);
			}
		});
		
		// behavior for the "send stored data" button
		((Button)findViewById(R.id.btnSendStoredData)).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				// zip all data to transfer: it creates only one file 
				File androidPhoneSignalDir = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal");
				File myReportDir = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport");
				String zipFileName = Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/archive_" + System.currentTimeMillis() + ".zip";
				ZipOutputStream zos = null;
				try {
					zos = new ZipOutputStream(new FileOutputStream(zipFileName));
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				}
				 try {
					 // lists all files in the AndroidPhoneSignal directory
				     for (int i = 0; i < androidPhoneSignalDir.list().length; i++) {
				    	 // if it is a file, add it to the archive
				    	 if (androidPhoneSignalDir.listFiles()[i].isFile() && !androidPhoneSignalDir.listFiles()[i].getName().endsWith(".zip"))
				    	 {
					         String filename = androidPhoneSignalDir.list()[i];
					         byte[] bytes = new byte[(int)androidPhoneSignalDir.listFiles()[i].length()];
					         new FileInputStream(androidPhoneSignalDir.listFiles()[i]).read(bytes);
					         ZipEntry entry = new ZipEntry(filename);
					         zos.putNextEntry(entry);
					         zos.write(bytes);
					         zos.closeEntry();
				    	 }
				    	 // if it is a directory, grab the files from that and put them in the archive
				    	 if (androidPhoneSignalDir.listFiles()[i].isDirectory())
				    	 {
				    		 File dir = androidPhoneSignalDir.listFiles()[i];
				    		 byte[] bytes;
				    		 ZipEntry entry;
					         for (int k = 0; k < dir.list().length; k++)
					         {
					        	 if (dir.listFiles()[k].isFile())
					        	 {
						        	 String filename = dir.list()[k];
							         bytes = new byte[(int)dir.listFiles()[k].length()];
							         new FileInputStream(dir.listFiles()[k]).read(bytes);
							         entry = new ZipEntry(dir.getName() + "/" + filename);
							         zos.putNextEntry(entry);
							         zos.write(bytes);
							         zos.closeEntry();
					        	 }
					         }
				    	 }
				     }
				 } catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
				     try {
						zos.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				 }
				
				// send the zipped data to the remote device
				File zipFile = new File(zipFileName);
				byte[] command = new byte[3];
				byte[] buffer = new byte[(int)zipFile.length()];
				FileInputStream fis = null;
				try {
					fis = new FileInputStream(zipFile);
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				}
				if (mySocket != null) {
					ConnectedThread ct = new ConnectedThread(mySocket);
					try {
						fis.read(buffer);
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					// write the "MultiPLe data" command for the server and send the zipped data
					ct.write("MPL".getBytes());
		        	ct.write(buffer);
		        	try {
						mySocket.getInputStream().read(command, 0, command.length);
					} catch (IOException e) {
						e.printStackTrace();
					}
		        	// close the connection if the server has received all the data
					if (new String(command).compareTo("END") == 0)
					{
						try {
							fis.close();
						} catch (IOException e1) {
							e1.printStackTrace();
						}
						ct.cancel();
						try {
							mySocket.close();
							mySocket = null;
						} catch (IOException e) {
							e.printStackTrace();
		        	}
						// delete all files transferred
						zipFile.delete();
						File[] app1 = androidPhoneSignalDir.listFiles();
						File[] app2 = myReportDir.listFiles();
						int l = app1.length;
						for (int i = 0; i < l; i++) {
							if (app1[i].isFile())
								app1[i].delete();
						}
						l = app2.length;
						for (int i = 0; i < l; i++) {
							app2[i].delete();
						}
					}
				}
			}
		});
		
        // behavior for the "Send current data" button
		((Button)findViewById(R.id.btnSendData)).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				byte[] buffer = new byte[4];
				if (mySocket != null) {
					ConnectedThread ct = new ConnectedThread(mySocket);
					// send SiNGle report, the one just acquired
					ct.write("SNG".getBytes());
		        	ct.write(jsonSignalReport.getBytes());
		        	try {
						mySocket.getInputStream().read(buffer, 0, buffer.length);
					} catch (IOException e) {
						e.printStackTrace();
					}
					if (new String(buffer).compareTo("END") == 0)
					{
						ct.cancel();
						try {
							mySocket.close();
							mySocket = null;
						} catch (IOException e) {
							e.printStackTrace();
		        	}
					}
				}
			}
		});
 		
        // action for clicking in the list of discovered devices
 		l.setOnItemClickListener(new OnItemClickListener() {
 		    public void onItemClick(AdapterView<?> parent, View view,
 		        int position, long id) {
 		      String text = (String) ((TextView) view).getText();
 		      Toast.makeText(getApplicationContext(), text, Toast.LENGTH_LONG).show();
 		      BluetoothDevice btd = BluetoothAdapter.getDefaultAdapter().getRemoteDevice(text.split("\n")[1]);
 		      ConnectThread ct = new ConnectThread(btd);
 		      ct.run();
 		    }
 		  });
	}
	
	// action for the bluetooth activation intent, based on the result
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == RESULT_OK) {
        	if (requestCode == REQUEST_ENABLE_BT) {
        		Set<BluetoothDevice> pairedDevices = myBluetoothAdapter.getBondedDevices();
                myArrayAdapter = new ArrayAdapter<String>(this,R.layout.list_item);
                 // Loop through paired devices
                 for (BluetoothDevice device : pairedDevices) {
                     // Add the name and address to an array adapter to show in a ListView
                     myArrayAdapter.add(device.getName() + "\n" + device.getAddress());
                 }
                 l.setAdapter(myArrayAdapter);
        	}	
        }
	}
	
	public void onDestroy() {
		super.onDestroy();
		myBluetoothAdapter.cancelDiscovery();
		unregisterReceiver(mReceiver);
	}
	
	private class AcceptThread extends Thread {
	    private final BluetoothServerSocket mmServerSocket;
	 
	    public AcceptThread() { 
	        // Use a temporary object that is later assigned to mmServerSocket,
	        // because mmServerSocket is final
	        BluetoothServerSocket tmp = null;
	        try {
	            // using UUID app string, also used by the client code
	            tmp = myBluetoothAdapter.listenUsingRfcommWithServiceRecord("AndroidPhoneSignal", uuid);
	        } catch (IOException e) { }
	        mmServerSocket = tmp;
	    }
	 
	    public void run() {
	        BluetoothSocket socket = null;
	        // Keep listening until exception occurs or a socket is returned
	        while (true) {
	            try {
	                socket = mmServerSocket.accept();
	                mySocket = socket;
					Log.d(TAG, "Connection accepted by server");
	            } catch (IOException e) {
	                break;
	            }
	            // If a connection was accepted
	            if (socket != null) {
	                // Do work to manage the connection (in a separate thread)
	                ConnectedThread ct = new ConnectedThread(socket);
	                ct.run();
	                try {
						mmServerSocket.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
	                break;
	            }
	        }
	    }
	 
	    // Will cancel the listening socket, and cause the thread to finish
	    @SuppressWarnings("unused")
		public void cancel() {
	        try {
	            mmServerSocket.close();
	        } catch (IOException e) { }
	    }
	}
	
	private class ConnectThread extends Thread {
	    private final BluetoothSocket mmSocket;
	    private final BluetoothDevice mmDevice;
	 
	    public ConnectThread(BluetoothDevice device) {
	        // Use a temporary object that is later assigned to mmSocket,
	        // because mmSocket is final
	        BluetoothSocket tmp = null;
	        mmDevice = device;
	 
	        // Get a BluetoothSocket to connect with the given BluetoothDevice
	        try {
	            // UUID string for the application, also used by the server code
	            tmp = device.createRfcommSocketToServiceRecord(uuid);
	        } catch (IOException e) { }
	        mmSocket = tmp;
	    }
	 
	    public void run() {
	        // Cancel discovery because it will slow down the connection
	        myBluetoothAdapter.cancelDiscovery();
	 
	        try {
	            // Connect the device through the socket. This will block
	            // until it succeeds or throws an exception
	            mmSocket.connect();
	            Toast.makeText(getApplicationContext(), "Connected to " + mmDevice.getName(), Toast.LENGTH_LONG).show();
	            mySocket = mmSocket;
	            Log.d(TAG, "Connection client ok");
	        } catch (IOException connectException) {
	            // Unable to connect; close the socket and get out
	            try {
	            	Log.d(TAG, connectException.getMessage());
	                mmSocket.close();
	                mySocket.close();
	            } catch (IOException closeException) {Log.d(TAG, "Connection client error"); }
	            return;
	        }
	 
	        // Put here work to manage the connection (in a separate thread)
	    }
	 
	    // Will cancel an in-progress connection, and close the socket
	    @SuppressWarnings("unused")
		public void cancel() {
	        try {
	            mmSocket.close();
	        } catch (IOException e) { }
	    }
	}
	
	// class for connection established
	private class ConnectedThread extends Thread {
	    private final BluetoothSocket mmSocket;
	    private final InputStream mmInStream;
	    private final OutputStream mmOutStream;
	 
	    public ConnectedThread(BluetoothSocket socket) {
	        mmSocket = socket;
	        InputStream tmpIn = null;
	        OutputStream tmpOut = null;
	        
	        // Get the input and output streams, using temp objects because
	        // member streams are final
	        try {
	            tmpIn = socket.getInputStream();
	            tmpOut = socket.getOutputStream();
	        } catch (IOException e) { }

	        mmInStream = tmpIn;
	        mmOutStream = tmpOut;
	    }
	    
	    public void run() {
	        byte[] buffer = new byte[1024];  // buffer store for the stream
	        byte[] command = new byte[3];
	        int bytes; // bytes returned from read()
	        File f = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/" + System.currentTimeMillis() + ".aps");
	        File archive = new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/archive_" + System.currentTimeMillis() + ".zip");
	        FileOutputStream fos = null;
	        try {
				mmInStream.read(command,0,command.length);
			} catch (IOException e2) {
				e2.printStackTrace();
			}
	        // case for single report
	        if (new String(command).compareTo("SNG") == 0) {
			try {
					fos = new FileOutputStream(f);
					bytes = mmInStream.read(buffer);
					fos.write(buffer, 0, bytes);
					mmOutStream.write("END".getBytes());
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			finally {
            	try {
					fos.close();
					mmInStream.close();
					mmOutStream.close();
					mmSocket.close();
					mySocket.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
	    }
	        // case for multiple reports
	        if (new String(command).compareTo("MPL") == 0) {
	        	byte[] bufferMPL = new byte[1000000];
				try {
					fos = new FileOutputStream(archive);
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				}
				// Keep listening to the InputStream
	        	try{
	                // Read from the InputStream
	        			bytes = mmInStream.read(bufferMPL);
	        			fos.write(bufferMPL, 0, bytes);
        				mmOutStream.write("END".getBytes());
	            } catch (IOException e) {
	                e.getMessage();
	            }
	            finally{
	            	try {
						fos.close();
						mmInStream.close();
						mmOutStream.close();
						mmSocket.close();
						mySocket.close();
						 ZipInputStream zis = new ZipInputStream(new FileInputStream(archive));
						     ZipEntry ze;
						     while ((ze = zis.getNextEntry()) != null && !ze.isDirectory()) {
						         ByteArrayOutputStream baos = new ByteArrayOutputStream();
						         buffer = new byte[1024];
						         int count;
						         while ((count = zis.read(buffer)) != -1) {
						             baos.write(buffer, 0, count);
						         }
						         String filename = ze.getName();
						         byte[] data = baos.toByteArray();
						         fos = new FileOutputStream(new File(Environment.getExternalStorageDirectory() + "/AndroidPhoneSignal/" + filename.split("/")[filename.split("/").length-1]));
						         fos.write(data);
						         fos.close();
						     }
						     archive.delete();
					} catch (IOException e) {
						e.printStackTrace();
					}
	            }
			}
	    }
	 
	    // write bytes on the output stream
	    public void write(byte[] bytes) {
	        try {
	            mmOutStream.write(bytes);
	        } catch (IOException e) { }
	    }
	 
	    // Call this from the main Activity to shutdown the connection
	    public void cancel() {
	        try {
	            mmSocket.close();
	        } catch (IOException e) { }
	    }
	}
}
