package com.androidphonesignal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpConnectionParams;

import com.google.gson.Gson;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.text.Html;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

public class ReportTransferActivity extends Activity{
	@SuppressWarnings("unused")
	private static final String TAG = "ReportTransferActivity";
	
	private Bundle extras;
	private TextView txtParameters;
	private String reportFileName;
	private boolean reportSaved;
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.report);
        reportSaved = false;
        extras = getIntent().getExtras();
        txtParameters = (TextView) findViewById(R.id.txtParameters);
        txtParameters.setText(Html.fromHtml("<b>REPORT:</b><br><br><br>"));
        if(extras !=null) {
        	txtParameters.append(Html.fromHtml("<b>Operator Name: </b>" + extras.getString("OperatorName") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Average signal strength: </b>" + extras.getInt("SignalStrength") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Network Type: </b>" + extras.getString("NetworkType") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Roaming: </b>" + extras.getBoolean("Roaming") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Last known CellID: </b>" + extras.getString("CellID") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Light info: </b>" + extras.getFloat("LightInfo") + (extras.getFloat("LightInfo")==-1.0 ? " (Not available)":"") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Latitude: </b>" + extras.getDouble("Latitude") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Longitude: </b>" + extras.getDouble("Longitude") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Location provider: </b>" + extras.getString("LocationProvider") + "<br>"));
        	txtParameters.append(Html.fromHtml("<b>Timestamp: </b>" + extras.getLong("Timestamp")));
        }
        if (new File(Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport/" + extras.getLong("Timestamp") + ".aps").exists()) {
        	reportFileName = Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport/" + extras.getLong("Timestamp") + ".aps";
        }
        // behavior for the bluetooth sharing button
        ((ImageButton) findViewById(R.id.imgBtnSend)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent i = new Intent(ReportTransferActivity.this, BluetoothActivity.class);
				i.putExtras(extras);
				startActivity(i);
			}
		});
        
        // behavior for the save data button
        ((ImageButton) findViewById(R.id.imgBtnSaveOwnData)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (reportFileName == null) {
					SignalReportTO mySignalReport = new SignalReportTO();
					String jsonSignalReport;
					Gson signalReportGson = new Gson();
					reportFileName = Environment.getExternalStorageDirectory().toString() + "/AndroidPhoneSignal/myReport/" + extras.getLong("Timestamp") + ".aps";
			        File f = new File(reportFileName);
			        FileOutputStream fos = null;
			        try {
						fos = new FileOutputStream(f);
					} catch (FileNotFoundException e) {
						e.printStackTrace();
					}
			        mySignalReport.setAvgSignalStrength(extras.getInt("SignalStrength"));
			        mySignalReport.setCellID(extras.getString("CellID"));
			        mySignalReport.setLatitude(extras.getDouble("Latitude"));
			        mySignalReport.setLightInfo(extras.getFloat("LightInfo"));
			        mySignalReport.setLongitude(extras.getDouble("Longitude"));
			        mySignalReport.setNetworkType(extras.getString("NetworkType"));
			        mySignalReport.setOperatorName(extras.getString("OperatorName"));
			        mySignalReport.setRoaming(extras.getBoolean("Roaming"));
			        mySignalReport.setLocationProvider(extras.getString("LocationProvider"));
			        mySignalReport.setTimestamp(extras.getLong("Timestamp"));
			        
			        // convert SignalReportTO object to a Json string
			        jsonSignalReport = signalReportGson.toJson(mySignalReport);
			        try {
			        	if (fos != null) {
			        		fos.write(jsonSignalReport.getBytes());
			        		fos.close();
			        		Toast.makeText(getApplicationContext(), "Data saved in " + f.getAbsolutePath(), Toast.LENGTH_LONG).show();
			        		reportSaved = true;
			        		extras.putString("reportFileName", reportFileName);
			        	}
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				else {
					reportSaved = true;
					Toast.makeText(getApplicationContext(), "Report already saved in file " + reportFileName +  "!", Toast.LENGTH_LONG).show();
				}
			}
		});
        
     // behavior for the save data button
        ((ImageButton) findViewById(R.id.imgBtnUpload)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (reportSaved) {
					HttpResponse resp;
					File f = new File(reportFileName);
					HttpClient httpClient = new DefaultHttpClient();
					httpClient.getParams().setParameter(HttpConnectionParams.CONNECTION_TIMEOUT, 10000);
					httpClient.getParams().setParameter(HttpConnectionParams.SO_TIMEOUT, 15000);
					httpClient.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);
					HttpPost httpPost = new HttpPost("http://192.168.0.3:8080/ProgettoJDBC/SrvHello");
					MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
					entity.addPart("reportFile", new FileBody(f));
					httpPost.setEntity(entity);
					Toast.makeText(getApplicationContext(), "Sending...", Toast.LENGTH_SHORT).show();
					try {
						resp = httpClient.execute(httpPost);
						if (resp.getStatusLine().getStatusCode() == 200) {
							f.delete();
							reportSaved = false;
						}
						Toast.makeText(getApplicationContext(), resp.getStatusLine().getStatusCode()==200?"File sent successfully!":"Transfer error. Retry!", Toast.LENGTH_LONG).show();
					} catch (ClientProtocolException e) {
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					} catch (ParseException e) {
						e.printStackTrace();
					} catch (IOException e) {
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					} catch (Exception e) {
						Toast.makeText(getApplicationContext(), e.getMessage() , Toast.LENGTH_LONG).show();
						e.printStackTrace();
					}
				 }
				 else {
					 Toast.makeText(getApplicationContext(), "Save the report before upload!", Toast.LENGTH_LONG).show();
				 }
			}
		});
	}
}
